/*
 * Copyright(c) 2006 IWAMURO Motonori <vmi@nifty.com>
 * 
 * Created: 2006/03/05
 */
package jp.vmi.util;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * operations of perl like s///e, s///ge
 */
public class RegexUtil {
    /**
     * replacement by code.
     */
    public interface Filter {
        /**
         * @param sb
         * @param matcher
         */
        void filter(StringBuffer sb, Matcher matcher);
    }

    /**
     * pattern = qr/PATTERN/;
     * string =~ s/pattern/filter/e;
     * 
     * @param string target string.
     * @param pattern compiled pattern of regular expression.
     * @param filter filter object.
     * @return replaced string.
     */
    public static String replaceFirst(String string, Pattern pattern, Filter filter) {
        Matcher matcher = pattern.matcher(string);
        matcher.reset();
        boolean result = matcher.find();
        if (result) {
            StringBuffer sb = new StringBuffer();
            sb.append(string.subSequence(0, matcher.start()));
            filter.filter(sb, matcher);
            sb.append(string.subSequence(matcher.end(), string.length()));
            return sb.toString();
        }
        return string;
    }

    /**
     * string =~ s/regex/filter/e;
     *
     * @param string target string.
     * @param regex pattern of regular expression.
     * @param filter filter object.
     * @return replaced string.
     */
    public static String replaceFirst(String string, String regex, Filter filter) {
        return replaceFirst(string, Pattern.compile(regex), filter);
    }

    /**
     * pattern = qr/PATTERN/;
     * string =~ s/pattern/filter/ge;
     * 
     * @param string target string.
     * @param pattern compiled pattern of regular expression.
     * @param filter filter object.
     * @return replaced string.
     */
    public static String replaceAll(String string, Pattern pattern, Filter filter) {
        Matcher matcher = pattern.matcher(string);
        matcher.reset();
        boolean result = matcher.find();
        if (result) {
            StringBuffer sb = new StringBuffer();
            int prevEnd = 0;
            do {
                int nextStart = matcher.start();
                if (prevEnd < nextStart)
                    sb.append(string.subSequence(prevEnd, nextStart));
                filter.filter(sb, matcher);
                prevEnd = matcher.end();
                result = matcher.find();
            } while (result);
            sb.append(string.subSequence(prevEnd, string.length()));
            return sb.toString();
        }
        return string;
    }

    /**
     * string =~ s/regex/filter/ge;
     * 
     * @param string target string.
     * @param regex pattern of regular expression.
     * @param filter filter class.
     * @return replaced string.
     */
    public static String replaceAll(String string, String regex, Filter filter) {
        return replaceAll(string, Pattern.compile(regex), filter);
    }
}
